@@ -1,5 +1,6 @@
 #' @include seurat.R
 NULL
+
 #' Cluster Determination
 #'
 #' Identify clusters of cells by a shared nearest neighbor (SNN) modularity
@@ -333,7 +334,6 @@ NumberClusters <- function(object) {
 #'
 #' @import Matrix
 #' @importFrom stats predict
-#' @importFrom ranger ranger
 #'
 #' @export
 #'
@@ -356,6 +356,7 @@ ClassifyCells <- function(
   new.data = NULL,
   ...
 ) {
+  PackageCheck('ranger')
   # build the classifier
   if (missing(classifier)){
     classifier <- BuildRFClassifier(
@@ -397,7 +398,6 @@ ClassifyCells <- function(
 #' @return Returns the random forest classifier
 #'
 #' @import Matrix
-#' @importFrom ranger ranger
 #'
 #' @export
 #'
@@ -414,6 +414,7 @@ BuildRFClassifier <- function(
   verbose = TRUE,
   ...
 ) {
+  PackageCheck('ranger')
   training.classes <- as.vector(x = training.classes)
   training.genes <- SetIfNull(
     x = training.genes,
@@ -430,7 +431,7 @@ BuildRFClassifier <- function(
   if (verbose) {
     message("Training Classifier ...")
   }
-  classifier <- ranger(
+  classifier <- ranger::ranger(
     data = training.data,
     dependent.variable.name = "class",
     classification = TRUE,
@@ -442,7 +443,7 @@ BuildRFClassifier <- function(
 
 #' K-Means Clustering
 #'
-#' Perform k=means clustering on both genes and single cells
+#' Perform k-means clustering on both genes and single cells
 #'
 #' K-means and heatmap are calculated on object@@scale.data
 #'
@@ -464,7 +465,6 @@ BuildRFClassifier <- function(
 #'
 #' @importFrom methods new
 #' @importFrom stats kmeans
-#' @importFrom tclust tkmeans
 #'
 #' @return Seurat object where the k-means results for genes is stored in
 #' object@@kmeans.obj[[1]], and the k-means results for cells is stored in
@@ -506,7 +506,8 @@ DoKMeans <- function(
   kmeans.data <- data.use[genes.use, cells.use]
   if (do.constrained) {
     set.seed(seed = k.seed)
-    kmeans.obj <- tkmeans(x = kmeans.data, k = k.genes, ...)
+    PackageCheck('tclust')
+    kmeans.obj <- tclust::tkmeans(x = kmeans.data, k = k.genes, ...)
   } else {
     set.seed(seed = k.seed)
     kmeans.obj <- kmeans(x = kmeans.data, centers = k.genes, ...)
