@@ -22,16 +22,16 @@ Eigen::SparseMatrix<double> RunUMISampling(Eigen::SparseMatrix<double> data, int
         if( (upsample) || (colSums[k] > sample_val)){
           entry = entry * double(sample_val) / colSums[k];
           if (fmod(entry, 1) != 0){
-            double rn = runif(1)[0];
+            double rn = R::runif(0,1);
             if(fmod(entry, 1) <= rn){
-              data.coeffRef(it.row(), it.col()) = floor(entry);
+              it.valueRef() = floor(entry);
             }
             else{
-              data.coeffRef(it.row(), it.col()) = ceil(entry);
+              it.valueRef() = ceil(entry);
             }
           }
           else{
-            data.coeffRef(it.row(), it.col()) = entry;
+            it.valueRef() = entry;
           }
         }
       }
@@ -50,16 +50,16 @@ Eigen::SparseMatrix<double> RunUMISamplingPerCell(Eigen::SparseMatrix<double> da
       if( (upsample) || (colSums[k] > sample_val[k])){
         entry = entry * double(sample_val[k]) / colSums[k];
         if (fmod(entry, 1) != 0){
-          double rn = runif(1)[0];
+          double rn = R::runif(0,1);
           if(fmod(entry, 1) <= rn){
-            data.coeffRef(it.row(), it.col()) = floor(entry);
+            it.valueRef() = floor(entry);
           }
           else{
-            data.coeffRef(it.row(), it.col()) = ceil(entry);
+            it.valueRef() = ceil(entry);
           }
         }
         else{
-          data.coeffRef(it.row(), it.col()) = entry;
+          it.valueRef() = entry;
         }
       }
     }
@@ -117,7 +117,7 @@ Eigen::SparseMatrix<double> LogNorm(Eigen::SparseMatrix<double> data, int scale_
   for (int k=0; k < data.outerSize(); ++k){
     p.increment();
     for (Eigen::SparseMatrix<double>::InnerIterator it(data, k); it; ++it){
-      data.coeffRef(it.row(), it.col()) = log1p(double(it.value()) / colSums[k] * scale_factor);
+      it.valueRef() = log1p(double(it.value()) / colSums[k] * scale_factor);
     }
   }
   return data;
